@@ -4,17 +4,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -37,7 +37,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -49,10 +49,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -96,68 +96,115 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_error.cc,v 1.10 2005/01/31 20:40:45 fyodor Exp $ */
+/* $Id: nmap_error.cc 3869 2006-08-25 01:47:49Z fyodor $ */
 
 #include "nmap_error.h"
+#include "output.h"
+#include "NmapOps.h"
+
+extern NmapOps o;
 
 #ifdef WIN32
 #include <windows.h>
 #endif /* WIN32 */
 
-#ifdef __amigaos__
-extern void CloseLibs(void);
-#endif
-
-void fatal(char *fmt, ...) {
-va_list  ap;
-va_start(ap, fmt);
-fflush(stdout);
-vfprintf(stderr, fmt, ap);
-fprintf(stderr, "\nQUITTING!\n");
-fflush(stderr);
-va_end(ap);
-exit(1);
+void fatal(const char *fmt, ...) {
+  va_list  ap;
+  va_start(ap, fmt);
+  log_vwrite(LOG_STDERR, fmt, ap);
+  va_end(ap);
+  if (o.log_errors) {
+    va_start(ap, fmt);
+    log_vwrite(LOG_NORMAL, fmt, ap);
+    va_end(ap);
+  }
+  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, "\nQUITTING!\n");
+  exit(1);
 }
 
-void error(char *fmt, ...) {
-va_list  ap;
-va_start(ap, fmt);
-fflush(stdout);
-vfprintf(stderr, fmt, ap);
-fprintf(stderr, "\n");
-fflush(stderr);
-va_end(ap);
-return;
+void error(const char *fmt, ...) {
+  va_list  ap;
+
+  va_start(ap, fmt);
+  log_vwrite(LOG_STDERR, fmt, ap);
+  va_end(ap);
+  
+  if (o.log_errors) {
+    va_start(ap, fmt);
+    log_vwrite(LOG_NORMAL, fmt, ap);
+    va_end(ap);
+  }
+  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, "\n");
+  return;
 }
 
-void pfatal(char *err, ...) {
+void pfatal(const char *err, ...) {
 #ifdef WIN32
-	int lasterror =0;
-	char *errstr = NULL;
+  int lasterror =0;
+  char *errstr = NULL;
 #endif
-	va_list  ap;va_start(ap, err);
-	fflush(stdout);
-	vfprintf(stderr, err, ap);
-	va_end(ap);
+  va_list ap;
+  
+  va_start(ap, err);
+  log_vwrite(LOG_STDERR, err, ap);
+  va_end(ap);
+
+  if (o.log_errors) {
+    va_start(ap, err);
+    log_vwrite(LOG_NORMAL, err, ap);
+    va_end(ap);
+  }
+
 #ifdef WIN32
-	lasterror = GetLastError();
-	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, NULL, lasterror, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+  lasterror = GetLastError();
+  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, 
+		NULL, lasterror, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
 		(LPTSTR) &errstr,  0, NULL);
-	fprintf(stderr, ": %s (%d)\n", errstr, lasterror);
-	HeapFree(GetProcessHeap(), 0, errstr);
+  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n", 
+	    errstr, lasterror);
+  HeapFree(GetProcessHeap(), 0, errstr);
 #else
-	perror(" ");
+  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n", 
+	    strerror(errno), errno);
 #endif /* WIN32 perror() compatability switch */
-	fflush(stderr);
-	exit(1);
+  if (o.log_errors) log_flush(LOG_STDOUT);
+  fflush(stderr);
+  exit(1);
 }
 
-void gh_perror(char *err, ...) {
-va_list  ap;va_start(ap, err);
-fflush(stdout);
-vfprintf(stderr, err, ap);
-va_end(ap);
-perror(" ");
-fflush(stderr);
-return;
+/* This function is the Nmap version of perror.  It is just copy and
+   pasted from pfatal(), except the exit has been replaced with a
+   return. */
+void gh_perror(const char *err, ...) {
+#ifdef WIN32
+  int lasterror =0;
+  char *errstr = NULL;
+#endif
+  va_list ap;
+  
+  va_start(ap, err);
+  log_vwrite(LOG_STDERR, err, ap);
+  va_end(ap);
+
+  if (o.log_errors) {
+    va_start(ap, err);
+    log_vwrite(LOG_NORMAL, err, ap);
+    va_end(ap);
+  }
+
+#ifdef WIN32
+  lasterror = GetLastError();
+  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, 
+		NULL, lasterror, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+		(LPTSTR) &errstr,  0, NULL);
+  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n", 
+	    errstr, lasterror);
+  HeapFree(GetProcessHeap(), 0, errstr);
+#else
+  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n", 
+	    strerror(errno), errno);
+#endif /* WIN32 perror() compatability switch */
+  if (o.log_errors) log_flush(LOG_STDOUT);
+  fflush(stderr);
+  return;
 }
