@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2006 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -53,7 +53,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_core.c 3439 2006-06-10 21:23:36Z fyodor $ */
+/* $Id: nsock_core.c 3870 2006-08-25 01:47:53Z fyodor $ */
 
 #include "nsock_internal.h"
 #include "gh_list.h"
@@ -547,9 +547,9 @@ void handle_read_result(mspool *ms, msevent *nse,
 			       enum nse_status status)  
 {
 
-  unsigned int count, len;
+  unsigned int count;
   char *str;
-  int rc;
+  int rc, len;
   msiod *iod = nse->iod;
 
   if (status == NSE_STATUS_TIMEOUT) {
