@@ -5,17 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -38,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -50,10 +50,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: portlist.cc,v 1.23 2004/11/12 20:25:06 fyodor Exp $ */
+/* $Id: portlist.cc 4068 2006-10-14 01:25:43Z fyodor $ */
 
 
 #include "portlist.h"
@@ -105,6 +105,8 @@
 #include "nmap.h"
 #include "NmapOps.h"
 
+using namespace std;
+
 #if HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
@@ -121,6 +123,7 @@ Port::Port() {
   serviceprobe_results = PROBESTATE_INITIAL;
   serviceprobe_service = NULL;
   serviceprobe_product = serviceprobe_version = serviceprobe_extrainfo = NULL;
+  serviceprobe_hostname = serviceprobe_ostype = serviceprobe_devicetype = NULL;
   serviceprobe_tunnel = SERVICE_TUNNEL_NONE;
   serviceprobe_fp = NULL;
 }
@@ -134,6 +137,12 @@ Port::~Port() {
    free(serviceprobe_version);
  if (serviceprobe_extrainfo)
    free(serviceprobe_extrainfo);
+ if (serviceprobe_hostname)
+   free(serviceprobe_hostname);
+ if (serviceprobe_ostype)
+   free(serviceprobe_ostype);
+ if (serviceprobe_devicetype)
+   free(serviceprobe_devicetype);
  if (serviceprobe_service)
    free(serviceprobe_service);
  if (serviceprobe_fp)
@@ -144,43 +153,33 @@ Port::~Port() {
 // out sd->fullversion.  If unavailable, it will be set to zero length.
 static void populateFullVersionString(struct serviceDeductions *sd) {
   char *dst = sd->fullversion;
-  char *end = sd->fullversion + sizeof(sd->fullversion);
-  int len;
+  unsigned int spaceleft = sizeof(sd->fullversion) - 1;
 
-  if (sd->product) {
-    len = strlen(sd->product);
-    len = MIN((int) sizeof(sd->fullversion) - 1, len);
-    memcpy(dst, sd->product, len);
-    dst += len;
+  dst[0] = '\0';
+
+  if (sd->product && spaceleft >= strlen(sd->product)) {
+    strncat(dst, sd->product, spaceleft);
+    spaceleft -= strlen(sd->product);
   }
 
-  if (sd->version && dst < end - 1) {
-    if (dst != sd->fullversion)
-      *(dst++) = ' ';
-    len = strlen(sd->version);
-    len = MIN(len, end - dst - 1);
-    memcpy(dst, sd->version, len);
-    dst += len;
+  if (sd->version && spaceleft >= (strlen(sd->version) + 1)) {
+    strncat(dst, " ", spaceleft);
+    strncat(dst, sd->version, spaceleft);
+    spaceleft -= strlen(sd->version) + 1;
   }
 
-  if (sd->extrainfo && dst < end) {
-    len = strlen(sd->extrainfo);
-    if (len < end - dst - 4) { // 4 == " ()\0"
-      if (dst != sd->fullversion)
-	*(dst++) = ' ';
-      *(dst++) = '(';
-      memcpy(dst, sd->extrainfo, len);
-      dst += len;
-      *(dst++) = ')';
-    }
+  if (sd->extrainfo && spaceleft >= (strlen(sd->extrainfo) + 3)) {
+    strncat(dst, " (", spaceleft);
+    strncat(dst, sd->extrainfo, spaceleft);
+    strncat(dst, ")", spaceleft);
+    spaceleft -= strlen(sd->extrainfo) + 3;
   }
 
-  *(dst++) = '\0'; // Will always have space
 }
 
 
-// pass in an allocated struct serviceDeductions (don't wory about
-// initializing, and you don't have to free any inernal ptrs.  See the
+// pass in an allocated struct serviceDeductions (don't worry about
+// initializing, and you don't have to free any internal ptrs.  See the
 // serviceDeductions definition for the fields that are populated.
 // Returns 0 if at least a name is available.
 int Port::getServiceDeductions(struct serviceDeductions *sd) {
@@ -203,6 +202,9 @@ int Port::getServiceDeductions(struct serviceDeductions *sd) {
     sd->dtype = SERVICE_DETECTION_PROBED; // RPC counts as probed
     sd->version = serviceprobe_version;
     sd->extrainfo = serviceprobe_extrainfo;
+    sd->hostname = serviceprobe_hostname;
+    sd->ostype = serviceprobe_ostype;
+    sd->devicetype = serviceprobe_devicetype;
     populateFullVersionString(sd);
     return 0;
   } else if (serviceprobe_results == PROBESTATE_FINISHED_HARDMATCHED
@@ -214,6 +216,19 @@ int Port::getServiceDeductions(struct serviceDeductions *sd) {
     sd->product = serviceprobe_product;
     sd->version = serviceprobe_version;
     sd->extrainfo = serviceprobe_extrainfo;
+    sd->hostname = serviceprobe_hostname;
+    sd->ostype = serviceprobe_ostype;
+    sd->devicetype = serviceprobe_devicetype;
+    populateFullVersionString(sd);
+    return 0;
+  } else if (serviceprobe_results == PROBESTATE_EXCLUDED) {
+    service = nmap_getservbyport(htons(portno), (proto == IPPROTO_TCP)? "tcp" : "udp");
+
+    if (service) sd->name = service->s_name;
+
+    sd->name_confidence = 2;  // Since we didn't even check it, we aren't very confident
+    sd->dtype = SERVICE_DETECTION_TABLE;
+    sd->product = serviceprobe_product;  // Should have a string that says port was excluded
     populateFullVersionString(sd);
     return 0;
   } else if (serviceprobe_results == PROBESTATE_FINISHED_TCPWRAPPED) {
@@ -243,65 +258,48 @@ int Port::getServiceDeductions(struct serviceDeductions *sd) {
 // will be NULL if unavailable. Note that this function makes its
 // own copy of sname and product/version/extrainfo.  This function
 // also takes care of truncating the version strings to a
-// 'reasonable' length if neccessary, and cleaning up any unprinable
+// 'reasonable' length if neccessary, and cleaning up any unprintable
 // chars. (these tests are to avoid annoying DOS (or other) attacks
 // by malicious services).  The fingerprint should be NULL unless
 // one is available and the user should submit it.  tunnel must be
 // SERVICE_TUNNEL_NULL (normal) or SERVICE_TUNNEL_SSL (means ssl was
 // detected and we tried to tunnel through it ).
+
+char* Port::cstringSanityCheck(const char* string, int len) {
+	char* result;
+  int slen;
+
+  if(!string)
+	  return NULL;
+
+  slen = strlen(string);
+  if (slen > len) slen = len;
+  result = (char *) safe_malloc(slen + 1);
+  memcpy(result, string, slen);
+  result[slen] = '\0';
+  replacenonprintable(result, slen, '.'); 
+  return result;
+}
+
 void Port::setServiceProbeResults(enum serviceprobestate sres, 
 				  const char *sname,	
 				  enum service_tunnel_type tunnel, 
 				  const char *product, const char *version, 
-				  const char *extrainfo,
+				  const char *extrainfo, const char *hostname,
+				  const char *ostype, const char *devicetype,
 				  const char *fingerprint) {
 
-  int slen;
   serviceprobe_results = sres;
-  unsigned char *p;
   serviceprobe_tunnel = tunnel;
   if (sname) serviceprobe_service = strdup(sname);
   if (fingerprint) serviceprobe_fp = strdup(fingerprint);
 
-  if (product) {
-    slen = strlen(product);
-    if (slen > 64) slen = 64;
-    serviceprobe_product = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_product, product, slen);
-    serviceprobe_product[slen] = '\0';
-    p = (unsigned char *) serviceprobe_product;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-
-  if (version) {
-    slen = strlen(version);
-    if (slen > 64) slen = 64;
-    serviceprobe_version = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_version, version, slen);
-    serviceprobe_version[slen] = '\0';
-    p = (unsigned char *) serviceprobe_version;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-
-  if (extrainfo) {
-    slen = strlen(extrainfo);
-    if (slen > 128) slen = 128;
-    serviceprobe_extrainfo = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_extrainfo, extrainfo, slen);
-    serviceprobe_extrainfo[slen] = '\0';
-    p = (unsigned char *) serviceprobe_extrainfo;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-
+	serviceprobe_product = cstringSanityCheck(product, 64);
+	serviceprobe_version = cstringSanityCheck(version, 64);
+	serviceprobe_extrainfo = cstringSanityCheck(extrainfo, 128);
+	serviceprobe_hostname = cstringSanityCheck(hostname, 64);
+	serviceprobe_ostype = cstringSanityCheck(ostype, 64);
+	serviceprobe_devicetype = cstringSanityCheck(devicetype, 64);
 }
 
 /* Sets the results of an RPC scan.  if rpc_status is not
@@ -341,58 +339,53 @@ void Port::setRPCProbeResults(int rpcs, unsigned long rpcp,
   }
 }
 
+/*****************************************************************************/
+/* Convert protocol name from in.h to enum portlist_proto.
+ * So IPPROTO_TCP will be changed to PORTLIST_PROTO_TCP and so on. */
+#define INPROTO2PORTLISTPROTO(p)		\
+  ((p)==IPPROTO_TCP ? PORTLIST_PROTO_TCP :	\
+   (p)==IPPROTO_UDP ? PORTLIST_PROTO_UDP :	\
+   PORTLIST_PROTO_IP)
+
+
 PortList::PortList() {
-  udp_ports = tcp_ports = ip_prots = NULL;
-  memset(state_counts, 0, sizeof(state_counts));
-  memset(state_counts_udp, 0, sizeof(state_counts_udp));
-  memset(state_counts_tcp, 0, sizeof(state_counts_tcp));
-  memset(state_counts_ip, 0, sizeof(state_counts_ip));
+  int proto;
+  memset(state_counts_proto, 0, sizeof(state_counts_proto));
+  memset(port_list, 0, sizeof(port_list));
+
+  for(proto=0; proto < PORTLIST_PROTO_MAX; proto++) {
+    if(port_list_count[proto] > 0)
+      port_list[proto] = (Port**) safe_zalloc(sizeof(Port*)*port_list_count[proto]);
+  }
+
   numports = 0;
   idstr = NULL;
 }
 
 PortList::~PortList() {
-  int i;
-
-  if (tcp_ports) {  
-    for(i=0; i < 65536; i++) {
-      if (tcp_ports[i])
-	delete tcp_ports[i];
-    }
-    free(tcp_ports);
-    tcp_ports = NULL;
-  }
-
-  if (udp_ports) {  
-    for(i=0; i < 65536; i++) {
-      if (udp_ports[i])
-	delete udp_ports[i];
-    }
-    free(udp_ports);
-    udp_ports = NULL;
-  }
-
-  if (ip_prots) {
-    for(i=0; i < 256; ++i) {
-      if (ip_prots[i])
-	delete ip_prots[i];
-    }
-    free(ip_prots);
-    ip_prots = NULL;
-  }
+  int proto, i;
 
   if (idstr) { 
     free(idstr);
     idstr = NULL;
   }
 
+  for(proto=0; proto < PORTLIST_PROTO_MAX; proto++) { // for every protocol
+    if(port_list[proto]) {
+      for(i=0; i < port_list_count[proto]; i++) { // free every Port 
+        if(port_list[proto][i]) 
+          delete port_list[proto][i];
+      }
+      free(port_list[proto]);
+    }
+  }
 }
 
 
 int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
-  Port *current = NULL;
-  Port **portarray = NULL;
+  Port *current;
   char msg[128];
+  int proto = INPROTO2PORTLISTPROTO(protocol);
 
   assert(state < PORT_HIGHEST_STATE);
 
@@ -405,99 +398,57 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
 	      statenum2str(state), portno, 
 	      proto2ascii(protocol), msg, idstr? idstr : "");
     log_flush(LOG_STDOUT);
-    
-    /* Write out add port messages for XML format so wrapper libraries
-       can use it and not have to parse LOG_STDOUT ;), which is a
-       pain! REMOVED now that Nmap scans multiple hosts in parallel.
-       This addport does not even tell which host the new port was
-       on. */    
-    //    log_write(LOG_XML, "<addport state=\"%s\" portid=\"%hu\" protocol=\"%s\" owner=\"%s\"/>\n", statenum2str(state), portno, proto2ascii(protocol), ((owner && *owner) ? owner : ""));
-    log_flush(LOG_XML); 
   }
 
 
-/* Make sure state is OK */
+  /* Make sure state is OK */
   if (state != PORT_OPEN && state != PORT_CLOSED && state != PORT_FILTERED &&
       state != PORT_UNFILTERED && state != PORT_OPENFILTERED && 
       state != PORT_CLOSEDFILTERED)
     fatal("addPort: attempt to add port number %d with illegal state %d\n", portno, state);
 
-  if (protocol == IPPROTO_TCP) {
-    if (!tcp_ports) {
-      tcp_ports = (Port **) safe_zalloc(65536 * sizeof(Port *));
-    }
-    portarray = tcp_ports;
-  } else if (protocol == IPPROTO_UDP) {
-    if (!udp_ports) {
-      udp_ports = (Port **) safe_zalloc(65536 * sizeof(Port *));
-    }
-    portarray = udp_ports;
-  } else if (protocol == IPPROTO_IP) {
-    assert(portno < 256);
-    if (!ip_prots) {
-      ip_prots = (Port **) safe_zalloc(256 * sizeof(Port *));
-    }
-    portarray = ip_prots;
-  } else fatal("addPort: attempted port insertion with invalid protocol");
+  assert(protocol!=IPPROTO_IP || portno<256);
 
-  if (portarray[portno]) {
+  current = getPortEntry(portno, protocol);
+  if (current) {
     /* We must discount our statistics from the old values.  Also warn
        if a complete duplicate */
-    current = portarray[portno];    
     if (o.debugging && current->state == state && (!owner || !*owner)) {
       error("Duplicate port (%hu/%s)\n", portno, proto2ascii(protocol));
     } 
-    state_counts[current->state]--;
-    if (current->proto == IPPROTO_TCP) {
-      state_counts_tcp[current->state]--;
-    } else if (current->proto == IPPROTO_UDP) {
-      state_counts_udp[current->state]--;
-    } else
-      state_counts_ip[current->state]--;
+    state_counts_proto[proto][current->state]--;
   } else {
-    portarray[portno] = new Port();
-    current = portarray[portno];
-    numports++;
+    current = new Port();
     current->portno = portno;
+    current->proto = protocol;
+    numports++;
+    
+    setPortEntry(portno, protocol, current);
   }
   
-  state_counts[state]++;
   current->state = state;
-  if (protocol == IPPROTO_TCP) {
-    state_counts_tcp[state]++;
-  } else if (protocol == IPPROTO_UDP) {
-    state_counts_udp[state]++;
-  } else
-    state_counts_ip[state]++;
-  current->proto = protocol;
+  state_counts_proto[proto][state]++;
 
   if (owner && *owner) {
     if (current->owner)
       free(current->owner);
     current->owner = strdup(owner);
   }
-
+  
   return 0; /*success */
 }
 
 int PortList::removePort(u16 portno, u8 protocol) {
   Port *answer = NULL;
 
-  if (protocol == IPPROTO_TCP && tcp_ports) {
-   answer = tcp_ports[portno];
-   tcp_ports[portno] = NULL;
-  }
-
-  if (protocol == IPPROTO_UDP && udp_ports) {  
-    answer = udp_ports[portno];
-    udp_ports[portno] = NULL;
-  } else if (protocol == IPPROTO_IP && ip_prots) {
-    answer = ip_prots[portno] = NULL;
-  }
+  printf("Removed %d\n", portno);
 
+  answer = getPortEntry(portno, protocol);
   if (!answer)
     return -1;
 
+  setPortEntry(portno, protocol, NULL);
+
   if (o.verbose) {  
     log_write(LOG_STDOUT, "Deleting port %hu/%s, which we thought was %s\n",
 	      portno, proto2ascii(answer->proto),
@@ -505,12 +456,16 @@ int PortList::removePort(u16 portno, u8 protocol) {
     log_flush(LOG_STDOUT);
   }    
 
+  /* Discount statistics */
+  state_counts_proto[INPROTO2PORTLISTPROTO(protocol)][answer->state]--;
+  numports--;
+
   delete answer;
   return 0;
 }
 
   /* Saves an identification string for the target containing these
-     ports (an IP addrss might be a good example, but set what you
+     ports (an IP address might be a good example, but set what you
      want).  Only used when printing new port updates.  Optional.  A
      copy is made. */
 void PortList::setIdStr(const char *id) {
@@ -523,94 +478,243 @@ void PortList::setIdStr(const char *id) {
   snprintf(idstr, len, " on %s", id);
 }
 
-Port *PortList::lookupPort(u16 portno, u8 protocol) {
 
-  if (protocol == IPPROTO_TCP && tcp_ports)
-    return tcp_ports[portno];
-
-  if (protocol == IPPROTO_UDP && udp_ports)
-    return udp_ports[portno];
-
-  if (protocol == IPPROTO_IP && ip_prots)
-    return ip_prots[portno];
-
-  return NULL;
+int PortList::getStateCounts(int protocol, int state){
+  return(state_counts_proto[INPROTO2PORTLISTPROTO(protocol)][state]);
 }
 
-int PortList::getIgnoredPortState() {
-  int ignored = PORT_UNKNOWN;
-  int ignoredNum = 0;
-  int i;
-  for(i=0; i < PORT_HIGHEST_STATE; i++) {
-    if (i == PORT_OPEN || i == PORT_UNKNOWN || i == PORT_TESTING || 
-	i == PORT_FRESH) continue; /* Cannot be ignored */
-    if (state_counts[i] > ignoredNum) {
-      ignored = i;
-      ignoredNum = state_counts[i];
-    }
-  }
-
-  if (state_counts[ignored] < 15)
-    ignored = PORT_UNKNOWN;
-
-  return ignored;
+int PortList::getStateCounts(int state){
+  int sum=0, proto;
+  for(proto=0; proto < PORTLIST_PROTO_MAX; proto++)
+    sum += state_counts_proto[proto][state];
+  return(sum);
 }
 
-/* A function for iterating through the ports.  Give NULL for the
+  /* A function for iterating through the ports.  Give NULL for the
    first "afterthisport".  Then supply the most recent returned port
    for each subsequent call.  When no more matching ports remain, NULL
    will be returned.  To restrict returned ports to just one protocol,
-   specify IPPROTO_TCP or IPPROTO_UDP for allowed_protocol.  A 0 for
-   allowed_protocol matches either.  allowed_state works in the same
-   fashion as allowed_protocol. This function returns ports in numeric
+   specify IPPROTO_TCP or IPPROTO_UDP for allowed_protocol. A TCPANDUDP
+   for allowed_protocol matches either. A 0 for allowed_state matches 
+   all possible states. This function returns ports in numeric
    order from lowest to highest, except that if you ask for both TCP &
    UDP, every TCP port will be returned before we start returning UDP
-   ports.  */
-
+   ports */
 Port *PortList::nextPort(Port *afterthisport, 
-			 u8 allowed_protocol, int allowed_state, 
-			 bool allow_portzero) {
-
-  /* These two are chosen because they come right "before" port 1/tcp */
-unsigned int current_portno = 0;
-unsigned int current_proto = IPPROTO_TCP;
-
-if (afterthisport) {
-  current_portno = afterthisport->portno;
-  current_proto = afterthisport->proto;  /* (afterthisport->proto == IPPROTO_TCP)? IPPROTO_TCP : IPPROTO_UDP; */
-  current_portno++; /* Start on the port after the one we were given */ 
-} 
-
- if (!allow_portzero && current_portno == 0) current_portno++;
-
-/* First we look for TCP ports ... */
-if (current_proto == IPPROTO_TCP) {
- if ((allowed_protocol == 0 || allowed_protocol == IPPROTO_TCP) && 
-    current_proto == IPPROTO_TCP && tcp_ports)
-  for(; current_portno < 65536; current_portno++) {
-    if (tcp_ports[current_portno] &&
-	(!allowed_state || tcp_ports[current_portno]->state == allowed_state))
-      return tcp_ports[current_portno];
+			 int allowed_protocol, int allowed_state) {
+  int proto;
+  int mapped_pno;
+  Port *port;
+  
+  if(afterthisport) {
+    proto = INPROTO2PORTLISTPROTO(afterthisport->proto);
+    assert(port_map[proto]!=NULL); // Hmm, it's not posible to handle port that doesn't have anything in map
+    assert(afterthisport->proto!=IPPROTO_IP || afterthisport->portno<256);
+    mapped_pno = port_map[proto][afterthisport->portno];
+    mapped_pno++; //  we're interested in next port after current
+  }else { // running for the first time
+    if(allowed_protocol == TCPANDUDP)	// if both protocols, then first search TCP
+      proto = INPROTO2PORTLISTPROTO(IPPROTO_TCP);
+    else
+      proto = INPROTO2PORTLISTPROTO(allowed_protocol);
+    mapped_pno = 0;
   }
+  
+  if(port_list[proto] != NULL) {
+    for(;mapped_pno < port_list_count[proto]; mapped_pno++) {
+      port = port_list[proto][mapped_pno];
+      if(port && (allowed_state==0 || port->state==allowed_state))
+        return(port);
+    }
+  }
+  
+  /* if all protocols, than after TCP search UDP */
+  if(allowed_protocol == TCPANDUDP && proto == INPROTO2PORTLISTPROTO(IPPROTO_TCP))
+    return(nextPort(NULL, IPPROTO_UDP, allowed_state));
+  
+  return(NULL); 
+}
+      
+Port *PortList::getPortEntry(u16 portno, u8 protocol) {
+  int proto = INPROTO2PORTLISTPROTO(protocol);
+  int mapped_pno;
 
-  /*  Uh-oh.  We have tried all tcp ports, lets move to udp */
-  current_portno = 0;
-  current_proto = IPPROTO_UDP;
+  assert(protocol!=IPPROTO_IP || portno<256);
+  if(port_map[proto]==NULL || port_list[proto]==NULL)
+    fatal("getPortEntry(%i,%i): you're trying to access uninitialized protocol", portno, protocol);
+  mapped_pno = port_map[proto][portno];
+
+  assert(mapped_pno < port_list_count[proto]);
+  assert(mapped_pno >= 0);
+  
+  /* The ugly hack: we allow only port 0 to be mapped to 0 position */
+  if(mapped_pno==0 && portno!=0) {
+    error("WARNING: getPortEntry(%i,%i): this port was not mapped", portno, protocol);
+    return(NULL);
+  }else
+    return(port_list[proto][mapped_pno]);
 }
 
-if ((allowed_protocol == 0 || allowed_protocol == IPPROTO_UDP) && 
-    current_proto == IPPROTO_UDP && udp_ports) {
-  for(; current_portno < 65536; current_portno++) {
-    if (udp_ports[current_portno] &&
-	(!allowed_state || udp_ports[current_portno]->state == allowed_state))
-      return udp_ports[current_portno];
+void PortList::setPortEntry(u16 portno, u8 protocol, Port *port) {
+  int proto = INPROTO2PORTLISTPROTO(protocol);
+  int mapped_pno;
+
+  assert(protocol!=IPPROTO_IP || portno<256);
+  if(port_map[proto]==NULL || port_list[proto]==NULL)
+    fatal("setPortEntry(%i,%i): you're trying to access uninitialized protocol", portno, protocol);
+  mapped_pno = port_map[proto][portno];
+
+  assert(mapped_pno < port_list_count[proto]);
+  assert(mapped_pno >= 0);
+  
+  /* The ugly hack: we allow only port 0 to be mapped to 0 position */
+  if(mapped_pno==0 && portno!=0) {
+    error("WARNING: setPortEntry(%i,%i): this port was not mapped", portno, protocol);
+    return;
+  }
+  
+  port_list[proto][mapped_pno] = port;
+}
+
+/* Just free memory used by PortList::port_map[]. Should be done somewhere 
+ * before closing nmap. */
+void PortList::freePortMap(){
+  int proto;
+  for(proto=0; proto < PORTLIST_PROTO_MAX; proto++)
+    if(port_map[proto]){
+      free(port_map[proto]);
+      port_map[proto] = NULL;
   }
 }
+  
 
-/*  No more ports */
-return NULL;
+u16 *PortList::port_map[PORTLIST_PROTO_MAX];
+int PortList::port_list_count[PORTLIST_PROTO_MAX];
+
+/* This function must be runned before any PortList object is created.
+ * It must be runned for every used protocol. The data in "ports" 
+ * should be sorted. */
+void PortList::initializePortMap(int protocol, u16 *ports, int portcount) {
+  int i;
+  int unused_zero;	// aren't we using 0 port?
+  int ports_max = (protocol == IPPROTO_IP) ? 256 : 65536;
+  int proto = INPROTO2PORTLISTPROTO(protocol);
+  
+  if(port_map[proto]!=NULL)
+    fatal("initializePortMap: portmap for protocol %i already initialized", protocol);
+
+  assert(port_list_count[proto]==0);
+  
+  /* this memory will never be freed, but this is the way it has to be. */
+  port_map[proto] = (u16*) safe_zalloc(sizeof(u16)*ports_max);
+
+  /* Is zero port to be unused? */
+  if(portcount==0 || ports[0]!=0)
+    unused_zero = 1;
+  else
+    unused_zero = 0;
+  
+  /* The ugly hack: if we don't use 0 port, than we need one more extra element. */
+  port_list_count[proto] = portcount + unused_zero;
+  
+  for(i=0; i < portcount; i++) {
+    /* The ugly hack: if we don't use 0 port, than we must start counting from 1 */
+    port_map[proto][ports[i]] = i + unused_zero; // yes, this is the key line
+  }
+  /* So now port_map should have such structure (lets scan 2nd,4th and 6th port):
+   * 	port_map[0,0,1,0,2,0,3,...]	        <- indexes to port_list structure
+   * 	port_list[0,port_2,port_4,port_6]
+   * But if we scan 0, 2, and 4 port:
+   * 	port_map[0,0,1,0,2,...]		// yes, this 0 in first place isn't mistake
+   * 	port_list[port_0,port_2,port_4] 
+   * And in both cases we scan three ports. Ugly, isn't it? :) */
 }
 
+  /* Cycles through the 0 or more "ignored" ports which should be
+   consolidated for Nmap output.  They are returned sorted by the
+   number of prots in the state, starting with the most common.  It
+   should first be called with PORT_UNKNOWN to obtain the most popular
+   ignored state (if any).  Then call with that state to get the next
+   most popular one.  Returns the state if there is one, but returns
+   PORT_UNKNOWN if there are no (more) states which qualify for
+   consolidation */
+int PortList::nextIgnoredState(int prevstate) {
+
+  int beststate = PORT_UNKNOWN;
+  
+  for(int state=0; state < PORT_HIGHEST_STATE; state++) {
+    /* The state must be ignored */
+    if (!isIgnoredState(state)) 
+      continue;
+
+    /* We can't give the same state again ... */
+    if (state == prevstate) continue;
+
+    /* If a previous state was given, we must have fewer ports than
+       that one, or be tied but be a larger state number */
+    if (prevstate != PORT_UNKNOWN && 
+	(getStateCounts(state) > getStateCounts(prevstate) ||
+	 (getStateCounts(state) == getStateCounts(prevstate) && state <= prevstate)))
+      continue;
+
+    /* We only qualify if we have more ports than the current best */
+    if (beststate != PORT_UNKNOWN && getStateCounts(beststate) >= getStateCounts(state))
+      continue;
+
+    /* Yay!  We found the best state so far ... */
+    beststate = state;
+  }
+
+  return beststate;
+}
+
+/* Returns true if a state should be ignored (consolidated), false otherwise */
+bool PortList::isIgnoredState(int state) {
+
+  if (o.debugging > 2)
+    return false;
+
+  if (state == PORT_OPEN || state == PORT_UNKNOWN || state == PORT_TESTING ||
+      state == PORT_FRESH)
+    return false; /* Cannot be ignored */
+
+  /* If openonly, we always ignore states that don't at least have open
+     as a possibility. */
+  if (o.openOnly() && state != PORT_OPENFILTERED && state != PORT_UNFILTERED 
+      && getStateCounts(state) > 0)
+    return true;
+
+  int max_per_state = 25; // Ignore states with more ports than this
+  /* We will show more ports when verbosity is requested */
+  if (o.verbose || o.debugging)
+    max_per_state *= (o.verbose + 20 * o.debugging);
+  
+  if (getStateCounts(state) > max_per_state)
+    return true;
+
+  return false;
+}
+
+int PortList::numIgnoredStates() {
+  int numstates = 0;
+  for(int state=0; state < PORT_HIGHEST_STATE; state++) {
+    if (isIgnoredState(state))
+      numstates++;
+  }
+  return numstates;
+}
+
+int PortList::numIgnoredPorts() {
+
+  int numports = 0;
+  for(int state=0; state < PORT_HIGHEST_STATE; state++) {
+    if (isIgnoredState(state))
+      numports += getStateCounts(state);
+  }
+  return numports;
+}
+
+
 // Move some popular TCP ports to the beginning of the portlist, because
 // that can speed up certain scans.  You should have already done any port
 // randomization, this should prevent the ports from always coming out in the
