@@ -7,17 +7,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -40,7 +40,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://insecure.org/nmap/ to download Nmap.                             *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -52,10 +52,10 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
@@ -99,7 +99,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: timing.h,v 1.11 2004/08/29 09:12:04 fyodor Exp $ */
+/* $Id: timing.h 3869 2006-08-25 01:47:49Z fyodor $ */
 
 #ifndef NMAP_TIMING_H
 #define NMAP_TIMING_H
@@ -124,7 +124,7 @@ void adjust_timeouts2(const struct timeval *sent,
 void adjust_timeouts(struct timeval sent, struct timeout_info *to);
 
 
-/* Sleeps if necessary to ensure that it isn't called twice withen less
+/* Sleeps if necessary to ensure that it isn't called twice within less
    time than o.send_delay.  If it is passed a non-null tv, the POST-SLEEP
    time is recorded in it */
 void enforce_scan_delay(struct timeval *tv);
@@ -135,7 +135,7 @@ class ScanProgressMeter {
   ScanProgressMeter(char *stypestr);
   ~ScanProgressMeter();
 /* Decides whether a timing report is likely to even be
-   printed.  There are stringint limitations on how often they are
+   printed.  There are stringent limitations on how often they are
    printed, as well as the verbosity level that must exist.  So you
    might as well check this before spending much time computing
    progress info.  now can be NULL if caller doesn't have the current
@@ -151,12 +151,20 @@ class ScanProgressMeter {
    or not a line was printed.*/
   bool printStatsIfNeccessary(double perc_done, const struct timeval *now);
 
+  /* Prints an estimate of when this scan will complete. */
+  bool printStats(double perc_done, const struct timeval *now);
+
+  /* Prints that this task is complete. */
+  bool endTask(const struct timeval *now, const char *additional_info) { return beginOrEndTask(now, additional_info, false); }
+
   struct timeval begin; /* When this ScanProgressMeter was instantiated */
  private:
   struct timeval last_print_test; /* Last time printStatsIfNeccessary was called */
   struct timeval last_print; /* The most recent time the ETC was printed */
-  struct timeval last_est; /* The latest PRINTED estimate */
   char *scantypestr;
+  struct timeval last_est; /* The latest PRINTED estimate */
+
+  bool beginOrEndTask(const struct timeval *now, const char *additional_info, bool beginning);
 };
 
 #endif /* NMAP_TIMING_H */
